package cn.com.qmth.examcloud.support.handler.richText;

import cn.com.qmth.examcloud.commons.util.RegExpUtil;
import cn.com.qmth.examcloud.support.handler.richText.bean.BlockBean;
import cn.com.qmth.examcloud.support.handler.richText.bean.SectionBean;
import cn.com.qmth.examcloud.support.handler.richText.bean.SectionCollectionBean;
import com.mysql.cj.util.StringUtils;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.safety.Whitelist;
import org.jsoup.select.Elements;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description 图片类型的文本处理器
 * @Author lideyin
 * @Date 2020/5/15 16:28
 * @Version 1.0
 */
public class ImageTextHandler implements RichTextHandler {
    @Override
    public SectionCollectionBean handle(String richText) {
        SectionCollectionBean sectionCollection = new SectionCollectionBean();
        List<SectionBean> sectionList = new ArrayList<>();
        SectionBean section = new SectionBean();
        List<BlockBean> blockList = new ArrayList<>();

        //图片文本部分
        String txtStr = getImgTxt(richText);
        if (!StringUtils.isNullOrEmpty(txtStr)) {
            BlockBean blockBean = new BlockBean();
            blockBean.setType("text");
            blockBean.setValue(txtStr);
            blockList.add(blockBean);
        }

        //图片部分
        if (validateImgAnswer(richText)) {
            String imgSrc = getImgSrc(richText);
            if (imgSrc.indexOf("|") > -1) {
                String[] imgAnswers = imgSrc.split("\\|");

                for (int i = 0; i < imgAnswers.length; i++) {
                    BlockBean blockBean = new BlockBean();
                    blockBean.setType("text");
                    blockBean.setValue(getPureUrl(imgAnswers[i]));

                    //又拍云图片的宽*高
                    String imgWH = RegExpUtil.find(imgAnswers[i], "(\\d+)x(\\d+)");
                    if (!StringUtils.isNullOrEmpty(imgWH)) {
                        Map<String, Object> paramMap = new HashMap<>();
                        paramMap.put("width", Integer.valueOf(imgWH.split("x")[0]));
                        paramMap.put("height", Integer.valueOf(imgWH.split("x")[1]));
                        blockBean.setParam(paramMap);
                    }

                    blockList.add(blockBean);
                }
            }
        }

        section.setBlocks(blockList);
        sectionList.add(section);
        sectionCollection.setSections(sectionList);

        return sectionCollection;

    }

    /**
     * 获取图片作答的文本部分
     *
     * @param studentAnswer
     * @return
     */
    private String getImgTxt(String studentAnswer) {
        if (!StringUtils.isNullOrEmpty(studentAnswer)) {
            return Jsoup.clean(studentAnswer, Whitelist.none());
        }

        return null;
    }

    /**
     * 校验图片答案格式是否正确
     *
     * @param studentAnswer
     * @return
     */
    private boolean validateImgAnswer(String studentAnswer) {
        if (StringUtils.isNullOrEmpty(studentAnswer)) {
            return false;
        }

        String regExp = "[\\s\\S]*(ftp|https?)\\:\\/\\/([\\w\\_\\-]+)\\.([\\w\\-]+[\\.]?)*[\\w]+\\.[a-zA-Z]{2,10}(.*)\\.(png|jpg|gif|jpeg).*[\\s\\S]*";
        return studentAnswer.matches(regExp);
    }

    /**
     * 获取图片作答图片路径
     *
     * @param studentAnswer
     * @return
     */
    private String getImgSrc(String studentAnswer) {
        if (StringUtils.isNullOrEmpty(studentAnswer)) {
            return studentAnswer;
        }

        //图片题特殊处理(因为图片作答题中有html标签，只需要取url即可)
        Document doc = Jsoup.parse(studentAnswer);
        Elements imgElements = doc.select("img[src]");

        String imgSrc = "";

        for (Element el : imgElements) {
            String src = el.attr("src");
            if (!StringUtils.isNullOrEmpty(src)) {
                imgSrc += src + "|";
            }
        }
        if (!StringUtils.isNullOrEmpty(imgSrc)) {
            return imgSrc.substring(0, imgSrc.length() - 1);
        }

        return imgSrc;
    }

    /**
     * 获取图片url中不带参数部分，去除地址中的多余后缀
     *
     * @param imgUrl 图片地址
     * @return
     */
    private String getPureUrl(String imgUrl) {
        if (StringUtils.isNullOrEmpty(imgUrl)) {
            return "";
        }

        String regExp = "(ftp|https?)\\:\\/\\/([\\w\\_\\-]+)\\.([\\w\\-]+[\\.]?)*[\\w]+\\.[a-zA-Z]{2,10}(.*)\\.(png|jpg|gif|jpeg)";
        return RegExpUtil.find(imgUrl, regExp);
    }
}
